/*
 * This file is part of the SPiC RobotArm (https://gitlab.cs.fau.de/i4/spic/robotarm).
 * Copyright (c) 2019 by Bernhard Heinloth
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef COM_H
#define COM_H

/*! \brief Initialize the serial communication struct
 */
void comInit();

/*! \brief Set the read timeout
 * \param args string with deci second value for Timeout
 * \return `true` if value could be parsed and set
 */
bool comTimeout(const char * args);

/*! \brief Set the baud rate
 * \param args string with baud rate (300 - 115200)
 * \return `true` if value could be parsed and set
 */
bool comBaudrate(const char * args);

/*! \brief Specify to communication settings
 * \param args string with data bits, parity and stop bits
 * \return `true` if values could be parsed and set
 */
bool comSettings(const char * args);

/*! \brief Establish connection
 * \param device path to serial device
 * \return `true` if connection could be established
 */
bool comConnect(const char * device);

/*! \brief Receive a character from the device
 * \return received character
 */
char comReceive();

/*! \brief Send a character to the device
 * \param character to send
 */
void comSend(const char data);

#endif
