/*
 * This file is part of the SPiC RobotArm (https://gitlab.cs.fau.de/i4/spic/robotarm).
 * Copyright (c) 2019 by Bernhard Heinloth
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <math.h>
#include <stdbool.h>
#include <stdio.h>
#include <unistd.h>

#include "coord.h"

polar_t cart2polar(double x, double y){
	polar_t out;
	out.p = sqrt(x*x + y*y);
	out.phi = out.p == 0 ? 0 : (acos(fmax(fmin(x / out.p, 1), -1)) * (y / out.p < 0 ? -1 : 1));

	return out;
}

cart_t polar2cart(double p, radian_t phi){
	cart_t out;
	out.x = p * cos(phi);
	out.y = p * sin(phi);
	return out;

}

degree_t rad2deg(radian_t angle){
	return angle * 180.0 / M_PI;
}


radian_t deg2rad(degree_t angle){
	return angle * M_PI / 180.0;
}

radian_t cosangle(double a, double b, double c){
	// cosine formula :
	//			c² = a² + b² - 2 * a * b * cos(gamma)
	// <=>  cos(gamma) = (a² + b² - c²) / (2 * a * b)
	if (a != 0 && b != 0) {
		double cos_gamma = (a*a + b*b - c*c)/(2*a*b);
		if (cos_gamma >= -1 && cos_gamma <= 1)
			return acos(cos_gamma);
	}

	return NAN; // Error
}
